@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -650,10 +650,10 @@ LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromString(const string& name_app
     }
 }
         
-LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromSignals(const std::string& name_app, tvec signals,
+LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromSignals(const string& name_app, tvec signals,
                                                         int argc, const char* argv[],
-                                                        const std::string& target,
-                                                        std::string& error_msg,
+                                                        const string& target,
+                                                        string& error_msg,
                                                         int opt_level)
 {
     LOCK_API
@@ -665,7 +665,7 @@ LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromSignals(const std::string& na
         argv1[argc1++] = "llvm";
         argv1[argc1++] = "-o";
         argv1[argc1++] = "string";
-            // Copy arguments
+        // Copy arguments
         for (int i = 0; i < argc; i++) {
             argv1[argc1++] = argv[i];
         }
@@ -691,10 +691,10 @@ LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromSignals(const std::string& na
     }
 }
         
-LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromBoxes(const std::string& name_app, Tree box,
+LIBFAUST_API llvm_dsp_factory* createDSPFactoryFromBoxes(const string& name_app, Tree box,
                                                       int argc, const char* argv[],
-                                                      const std::string& target,
-                                                      std::string& error_msg,
+                                                      const string& target,
+                                                      string& error_msg,
                                                       int opt_level)
 {
     LOCK_API
