@@ -4,16 +4,16 @@
     Copyright (C) 2003-2018 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
@@ -34,12 +34,21 @@
 
 LIBFAUST_API Tree sigWriteReadTable(Tree n, Tree init, Tree widx, Tree wsig, Tree ridx)
 {
-    return sigRDTbl(sigWRTbl(gGlobal->nil, sigTable(gGlobal->nil, n, sigGen(init)), widx, wsig), ridx);
+    /*
+     rwtable are parsed as boxPrim5, so do not have a special treatment in eval/propagate. So we do here:
+     - the size argument is supposed to be known at compile time, so is casted at compilation time to int
+     */
+    return sigRDTbl(sigWRTbl(gGlobal->nil, sigTable(gGlobal->nil, sigInt(tree2int(n)), sigGen(init)), widx, wsig),
+                    ridx);
 }
 
 LIBFAUST_API Tree sigReadOnlyTable(Tree n, Tree init, Tree ridx)
 {
-    return sigRDTbl(sigTable(gGlobal->nil, n, sigGen(init)), ridx);
+    /*
+     rtable are parsed as boxPrim3, so do not have a special treatment in eval/propagate. So we do here:
+     - the size argument is supposed to be known at compile time, so is casted at compilation time to int
+     */
+    return sigRDTbl(sigTable(gGlobal->nil, sigInt(tree2int(n)), sigGen(init)), ridx);
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -48,7 +57,7 @@ LIBFAUST_API Tree sigReadOnlyTable(Tree n, Tree init, Tree ridx)
  */
 ///////////////////////////////////////////////////////////////////////
 
-Tree sigRem(Tree x, Tree y)
+LIBFAUST_API Tree sigRem(Tree x, Tree y)
 {
     if (isZero(y)) {
         stringstream error;
@@ -62,7 +71,7 @@ LIBFAUST_API Tree sigInt(int i)
 {
     return tree(i);
 }
-bool isSigInt(Tree t, int* i)
+LIBFAUST_API bool isSigInt(Tree t, int* i)
 {
     return isInt(t->node(), i);
 }
@@ -71,18 +80,17 @@ LIBFAUST_API Tree sigReal(double r)
 {
     return tree(r);
 }
-bool isSigReal(Tree t, double* r)
+LIBFAUST_API bool isSigReal(Tree t, double* r)
 {
     return isDouble(t->node(), r);
 }
 
 LIBFAUST_API Tree sigInput(int i)
 {
-    // Keep the max input number (used with the signal API)
-    gGlobal->gMaxInputs = std::max(gGlobal->gMaxInputs, i+1);
     return tree(gGlobal->SIGINPUT, tree(i));
 }
-bool isSigInput(Tree t, int* i)
+
+LIBFAUST_API bool isSigInput(Tree t, int* i)
 {
     Tree x;
     return isTree(t, gGlobal->SIGINPUT, x) && isInt(x->node(), i);
@@ -92,7 +100,7 @@ Tree sigOutput(int i, Tree t0)
 {
     return tree(gGlobal->SIGOUTPUT, tree(i), t0);
 }
-bool isSigOutput(Tree t, int* i, Tree& t0)
+LIBFAUST_API bool isSigOutput(Tree t, int* i, Tree& t0)
 {
     Tree x;
     return isTree(t, gGlobal->SIGOUTPUT, x, t0) && isInt(x->node(), i);
@@ -107,16 +115,16 @@ Tree sigDelay1(Tree t0)
 {
     return tree(gGlobal->SIGDELAY1, t0);
 }
-bool isSigDelay1(Tree t, Tree& t0)
+LIBFAUST_API bool isSigDelay1(Tree t, Tree& t0)
 {
     return isTree(t, gGlobal->SIGDELAY1, t0);
 }
 
 LIBFAUST_API Tree sigDelay(Tree t0, Tree t1)
 {
-    return tree(gGlobal->SIGDELAY, t0, sigIntCast(t1));
+    return tree(gGlobal->SIGDELAY, t0, t1);
 }
-bool isSigDelay(Tree t, Tree& t0, Tree& t1)
+LIBFAUST_API bool isSigDelay(Tree t, Tree& t0, Tree& t1)
 {
     return isTree(t, gGlobal->SIGDELAY, t0, t1);
 }
@@ -125,36 +133,36 @@ Tree sigPrefix(Tree t0, Tree t1)
 {
     return tree(gGlobal->SIGPREFIX, t0, t1);
 }
-bool isSigPrefix(Tree t, Tree& t0, Tree& t1)
+LIBFAUST_API bool isSigPrefix(Tree t, Tree& t0, Tree& t1)
 {
     return isTree(t, gGlobal->SIGPREFIX, t0, t1);
 }
 
 // Read only and read write tables
 
-Tree sigRDTbl(Tree t, Tree i)
+Tree sigRDTbl(Tree tb, Tree ri)
 {
-    return tree(gGlobal->SIGRDTBL, t, i);
+    return tree(gGlobal->SIGRDTBL, tb, ri);
 }
-bool isSigRDTbl(Tree s, Tree& t, Tree& i)
+LIBFAUST_API bool isSigRDTbl(Tree s, Tree& tb, Tree& ri)
 {
-    return isTree(s, gGlobal->SIGRDTBL, t, i);
+    return isTree(s, gGlobal->SIGRDTBL, tb, ri);
 }
 
-Tree sigWRTbl(Tree id, Tree t, Tree i, Tree s)
+Tree sigWRTbl(Tree id, Tree tb, Tree wi, Tree ws)
 {
-    return tree(gGlobal->SIGWRTBL, id, t, i, s);
+    return tree(gGlobal->SIGWRTBL, id, tb, wi, ws);
 }
-bool isSigWRTbl(Tree u, Tree& id, Tree& t, Tree& i, Tree& s)
+LIBFAUST_API bool isSigWRTbl(Tree u, Tree& id, Tree& tb, Tree& wi, Tree& ws)
 {
-    return isTree(u, gGlobal->SIGWRTBL, id, t, i, s);
+    return isTree(u, gGlobal->SIGWRTBL, id, tb, wi, ws);
 }
 
 Tree sigTable(Tree id, Tree n, Tree sig)
 {
     return tree(gGlobal->SIGTABLE, id, n, sig);
 }
-bool isSigTable(Tree t, Tree& id, Tree& n, Tree& sig)
+LIBFAUST_API bool isSigTable(Tree t, Tree& id, Tree& n, Tree& sig)
 {
     return isTree(t, gGlobal->SIGTABLE, id, n, sig);
 }
@@ -165,7 +173,7 @@ Tree sigGen(Tree s)
 {
     return tree(gGlobal->SIGGEN, s);
 }
-bool isSigGen(Tree t, Tree& x)
+LIBFAUST_API bool isSigGen(Tree t, Tree& x)
 {
     return isTree(t, gGlobal->SIGGEN, x);
 }
@@ -180,7 +188,7 @@ Tree sigDocConstantTbl(Tree n, Tree sig)
 {
     return tree(gGlobal->SIGDOCONSTANTTBL, n, sig);
 }
-bool isSigDocConstantTbl(Tree t, Tree& n, Tree& sig)
+LIBFAUST_API bool isSigDocConstantTbl(Tree t, Tree& n, Tree& sig)
 {
     return isTree(t, gGlobal->SIGDOCONSTANTTBL, n, sig);
 }
@@ -189,7 +197,7 @@ Tree sigDocWriteTbl(Tree n, Tree sig, Tree widx, Tree wsig)
 {
     return tree(gGlobal->SIGDOCWRITETBL, n, sig, widx, wsig);
 }
-bool isSigDocWriteTbl(Tree t, Tree& n, Tree& sig, Tree& widx, Tree& wsig)
+LIBFAUST_API bool isSigDocWriteTbl(Tree t, Tree& n, Tree& sig, Tree& widx, Tree& wsig)
 {
     return isTree(t, gGlobal->SIGDOCWRITETBL, n, sig, widx, wsig);
 }
@@ -198,7 +206,7 @@ Tree sigDocAccessTbl(Tree tbl, Tree ridx)
 {
     return tree(gGlobal->SIGDOCACCESSTBL, tbl, ridx);
 }
-bool isSigDocAccessTbl(Tree t, Tree& tbl, Tree& ridx)
+LIBFAUST_API bool isSigDocAccessTbl(Tree t, Tree& tbl, Tree& ridx)
 {
     return isTree(t, gGlobal->SIGDOCACCESSTBL, tbl, ridx);
 }
@@ -207,9 +215,9 @@ bool isSigDocAccessTbl(Tree t, Tree& tbl, Tree& ridx)
 
 LIBFAUST_API Tree sigSelect2(Tree selector, Tree s1, Tree s2)
 {
-    return tree(gGlobal->SIGSELECT2, sigIntCast(selector), s1, s2);
+    return tree(gGlobal->SIGSELECT2, selector, s1, s2);
 }
-bool isSigSelect2(Tree t, Tree& selector, Tree& s1, Tree& s2)
+LIBFAUST_API bool isSigSelect2(Tree t, Tree& selector, Tree& s1, Tree& s2)
 {
     return isTree(t, gGlobal->SIGSELECT2, selector, s1, s2);
 }
@@ -217,8 +225,7 @@ bool isSigSelect2(Tree t, Tree& selector, Tree& s1, Tree& s2)
 //  "select3" expressed with "select2"
 LIBFAUST_API Tree sigSelect3(Tree selector, Tree s1, Tree s2, Tree s3)
 {
-    return sigSelect2(sigBinOp(kEQ, sigIntCast(selector), sigInt(0)),
-                      sigSelect2(sigBinOp(kEQ, sigIntCast(selector), sigInt(1)), s3, s2), s1);
+    return sigSelect2(sigBinOp(kEQ, selector, sigInt(0)), sigSelect2(sigBinOp(kEQ, selector, sigInt(1)), s3, s2), s1);
 }
 
 Tree sigAssertBounds(Tree s1, Tree s2, Tree s3)
@@ -226,7 +233,7 @@ Tree sigAssertBounds(Tree s1, Tree s2, Tree s3)
     return tree(gGlobal->SIGASSERTBOUNDS, s1, s2, s3);
 }
 
-bool isSigAssertBounds(Tree t, Tree& s1, Tree& s2, Tree& s3)
+LIBFAUST_API bool isSigAssertBounds(Tree t, Tree& s1, Tree& s2, Tree& s3)
 {
     return isTree(t, gGlobal->SIGASSERTBOUNDS, s1, s2, s3);
 }
@@ -236,7 +243,7 @@ Tree sigHighest(Tree s)
     return tree(gGlobal->SIGHIGHEST, s);
 }
 
-bool isSigHighest(Tree t, Tree& s)
+LIBFAUST_API bool isSigHighest(Tree t, Tree& s)
 {
     return isTree(t, gGlobal->SIGHIGHEST, s);
 }
@@ -246,18 +253,24 @@ Tree sigLowest(Tree s)
     return tree(gGlobal->SIGLOWEST, s);
 }
 
-bool isSigLowest(Tree t, Tree& s)
+LIBFAUST_API bool isSigLowest(Tree t, Tree& s)
 {
     return isTree(t, gGlobal->SIGLOWEST, s);
 }
 
 // Arithmetical operations
 
+LIBFAUST_API Tree sigBinOp(SOperator op, Tree x, Tree y)
+{
+    return sigBinOp(int(op), x, y);
+}
+
 LIBFAUST_API Tree sigBinOp(int op, Tree x, Tree y)
 {
     return tree(gGlobal->SIGBINOP, tree(op), x, y);
 }
-bool isSigBinOp(Tree s, int* op, Tree& x, Tree& y)
+
+LIBFAUST_API bool isSigBinOp(Tree s, int* op, Tree& x, Tree& y)
 {
     Tree t;
     return isTree(s, gGlobal->SIGBINOP, t, x, y) && isInt(t->node(), op);
@@ -269,7 +282,7 @@ Tree sigFFun(Tree ff, Tree largs)
 {
     return tree(gGlobal->SIGFFUN, ff, largs);
 }
-bool isSigFFun(Tree s, Tree& ff, Tree& largs)
+LIBFAUST_API bool isSigFFun(Tree s, Tree& ff, Tree& largs)
 {
     return isTree(s, gGlobal->SIGFFUN, ff, largs);
 }
@@ -283,7 +296,7 @@ bool isSigFConst(Tree s)
     Tree t, n, f;
     return isTree(s, gGlobal->SIGFCONST, t, n, f);
 }
-bool isSigFConst(Tree s, Tree& type, Tree& name, Tree& file)
+LIBFAUST_API bool isSigFConst(Tree s, Tree& type, Tree& name, Tree& file)
 {
     return isTree(s, gGlobal->SIGFCONST, type, name, file);
 }
@@ -297,7 +310,7 @@ bool isSigFVar(Tree s)
     Tree t, n, f;
     return isTree(s, gGlobal->SIGFVAR, t, n, f);
 }
-bool isSigFVar(Tree s, Tree& type, Tree& name, Tree& file)
+LIBFAUST_API bool isSigFVar(Tree s, Tree& type, Tree& name, Tree& file)
 {
     return isTree(s, gGlobal->SIGFVAR, type, name, file);
 }
@@ -308,7 +321,7 @@ Tree sigProj(int i, Tree rgroup)
 {
     return tree(gGlobal->SIGPROJ, tree(i), rgroup);
 }
-bool isProj(Tree t, int* i, Tree& rgroup)
+LIBFAUST_API bool isProj(Tree t, int* i, Tree& rgroup)
 {
     Tree x;
     return isTree(t, gGlobal->SIGPROJ, x, rgroup) && isInt(x->node(), i);
@@ -318,40 +331,34 @@ bool isProj(Tree t, int* i, Tree& rgroup)
 
 LIBFAUST_API Tree sigIntCast(Tree t)
 {
-    Node n = t->node();
-
-    int i;
-    if (isInt(n, &i)) return t;
+    Node   n = t->node();
+    int    i;
     double x;
+
+    if (isInt(n, &i)) return t;
     if (isDouble(n, &x)) return tree(int(x));
-    if (isSigIntCast(t)) return t;
 
     return tree(gGlobal->SIGINTCAST, t);
 }
 
 LIBFAUST_API Tree sigFloatCast(Tree t)
 {
-    // cerr << "sigFloatCast(" << ppsig(t) << ")" << endl;
-    Node n = t->node();
-
-    int i;
-    if (isInt(n, &i)) return tree(double(i));
+    Node   n = t->node();
+    int    i;
     double x;
+
+    if (isInt(n, &i)) return tree(double(i));
     if (isDouble(n, &x)) return t;
-    if (isSigFloatCast(t)) return t;
-    if (isSigInput(t, &i)) return t;
 
     return tree(gGlobal->SIGFLOATCAST, t);
 }
 
-// Tree sigFloatCast(Tree t) { return isSigFloatCast(t)? t : tree(gGlobal->SIGFLOATCAST, t); }
-
 bool isSigIntCast(Tree t)
 {
     Tree x;
     return isTree(t, gGlobal->SIGINTCAST, x);
 }
-bool isSigIntCast(Tree t, Tree& x)
+LIBFAUST_API bool isSigIntCast(Tree t, Tree& x)
 {
     return isTree(t, gGlobal->SIGINTCAST, x);
 }
@@ -361,7 +368,7 @@ bool isSigFloatCast(Tree t)
     Tree x;
     return isTree(t, gGlobal->SIGFLOATCAST, x);
 }
-bool isSigFloatCast(Tree t, Tree& x)
+LIBFAUST_API bool isSigFloatCast(Tree t, Tree& x)
 {
     return isTree(t, gGlobal->SIGFLOATCAST, x);
 }
@@ -383,7 +390,6 @@ LIBFAUST_API Tree sigDiv(Tree x, Tree y)
 {
     return sigBinOp(kDiv, x, y);
 }
-Tree sigRem(Tree x, Tree y);
 
 LIBFAUST_API Tree sigAND(Tree x, Tree y)
 {
@@ -449,7 +455,7 @@ bool isSigButton(Tree s)
     Tree lbl;
     return isTree(s, gGlobal->SIGBUTTON, lbl);
 }
-bool isSigButton(Tree s, Tree& lbl)
+LIBFAUST_API bool isSigButton(Tree s, Tree& lbl)
 {
     return isTree(s, gGlobal->SIGBUTTON, lbl);
 }
@@ -463,7 +469,7 @@ bool isSigCheckbox(Tree s)
     Tree lbl;
     return isTree(s, gGlobal->SIGCHECKBOX, lbl);
 }
-bool isSigCheckbox(Tree s, Tree& lbl)
+LIBFAUST_API bool isSigCheckbox(Tree s, Tree& lbl)
 {
     return isTree(s, gGlobal->SIGCHECKBOX, lbl);
 }
@@ -472,7 +478,7 @@ LIBFAUST_API Tree sigWaveform(const tvec& wf)
 {
     return tree(gGlobal->SIGWAVEFORM, wf);
 }
-bool isSigWaveform(Tree s)
+LIBFAUST_API bool isSigWaveform(Tree s)
 {
     return isTree(s, gGlobal->SIGWAVEFORM);
 }
@@ -487,11 +493,11 @@ bool isSigHSlider(Tree s)
     return isTree(s, gGlobal->SIGHSLIDER, lbl, params);
 }
 
-bool isSigHSlider(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
+LIBFAUST_API bool isSigHSlider(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
 {
     Tree params;
     if (isTree(s, gGlobal->SIGHSLIDER, lbl, params)) {
-        init  = nth(params, 0);
+        init = nth(params, 0);
         min  = nth(params, 1);
         max  = nth(params, 2);
         step = nth(params, 3);
@@ -511,11 +517,11 @@ bool isSigVSlider(Tree s)
     return isTree(s, gGlobal->SIGVSLIDER, lbl, params);
 }
 
-bool isSigVSlider(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
+LIBFAUST_API bool isSigVSlider(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
 {
     Tree params;
     if (isTree(s, gGlobal->SIGVSLIDER, lbl, params)) {
-        init  = nth(params, 0);
+        init = nth(params, 0);
         min  = nth(params, 1);
         max  = nth(params, 2);
         step = nth(params, 3);
@@ -535,11 +541,11 @@ bool isSigNumEntry(Tree s)
     return isTree(s, gGlobal->SIGNUMENTRY, lbl, params);
 }
 
-bool isSigNumEntry(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
+LIBFAUST_API bool isSigNumEntry(Tree s, Tree& lbl, Tree& init, Tree& min, Tree& max, Tree& step)
 {
     Tree params;
     if (isTree(s, gGlobal->SIGNUMENTRY, lbl, params)) {
-        init  = nth(params, 0);
+        init = nth(params, 0);
         min  = nth(params, 1);
         max  = nth(params, 2);
         step = nth(params, 3);
@@ -560,7 +566,7 @@ bool isSigHBargraph(Tree s)
     Tree lbl, min, max, x;
     return isTree(s, gGlobal->SIGHBARGRAPH, lbl, min, max, x);
 }
-bool isSigHBargraph(Tree s, Tree& lbl, Tree& min, Tree& max, Tree& x)
+LIBFAUST_API bool isSigHBargraph(Tree s, Tree& lbl, Tree& min, Tree& max, Tree& x)
 {
     return isTree(s, gGlobal->SIGHBARGRAPH, lbl, min, max, x);
 }
@@ -574,7 +580,7 @@ bool isSigVBargraph(Tree s)
     Tree lbl, min, max, x;
     return isTree(s, gGlobal->SIGVBARGRAPH, lbl, min, max, x);
 }
-bool isSigVBargraph(Tree s, Tree& lbl, Tree& min, Tree& max, Tree& x)
+LIBFAUST_API bool isSigVBargraph(Tree s, Tree& lbl, Tree& min, Tree& max, Tree& x)
 {
     return isTree(s, gGlobal->SIGVBARGRAPH, lbl, min, max, x);
 }
@@ -583,7 +589,7 @@ Tree sigAttach(Tree t0, Tree t1)
 {
     return tree(gGlobal->SIGATTACH, t0, t1);
 }
-bool isSigAttach(Tree t, Tree& t0, Tree& t1)
+LIBFAUST_API bool isSigAttach(Tree t, Tree& t0, Tree& t1)
 {
     return isTree(t, gGlobal->SIGATTACH, t0, t1);
 }
@@ -592,7 +598,7 @@ Tree sigEnable(Tree t0, Tree t1)
 {
     return tree(gGlobal->SIGENABLE, t0, t1);
 }
-bool isSigEnable(Tree t, Tree& t0, Tree& t1)
+LIBFAUST_API bool isSigEnable(Tree t, Tree& t0, Tree& t1)
 {
     return isTree(t, gGlobal->SIGENABLE, t0, t1);
 }
@@ -601,7 +607,7 @@ Tree sigControl(Tree t0, Tree t1)
 {
     return tree(gGlobal->SIGCONTROL, t0, t1);
 }
-bool isSigControl(Tree t, Tree& t0, Tree& t1)
+LIBFAUST_API bool isSigControl(Tree t, Tree& t0, Tree& t1)
 {
     return isTree(t, gGlobal->SIGCONTROL, t0, t1);
 }
@@ -832,19 +838,19 @@ LIBFAUST_API Tree sigSoundfileBuffer(Tree sf, Tree chan, Tree part, Tree ridx)
     return tree(gGlobal->SIGSOUNDFILEBUFFER, sf, chan, part, ridx);
 }
 
-bool isSigSoundfile(Tree s, Tree& label)
+LIBFAUST_API bool isSigSoundfile(Tree s, Tree& label)
 {
     return isTree(s, gGlobal->SIGSOUNDFILE, label);
 }
-bool isSigSoundfileLength(Tree s, Tree& sf, Tree& part)
+LIBFAUST_API bool isSigSoundfileLength(Tree s, Tree& sf, Tree& part)
 {
     return isTree(s, gGlobal->SIGSOUNDFILELENGTH, sf, part);
 }
-bool isSigSoundfileRate(Tree s, Tree& sf, Tree& part)
+LIBFAUST_API bool isSigSoundfileRate(Tree s, Tree& sf, Tree& part)
 {
     return isTree(s, gGlobal->SIGSOUNDFILERATE, sf, part);
 }
-bool isSigSoundfileBuffer(Tree s, Tree& sf, Tree& chan, Tree& part, Tree& ridx)
+LIBFAUST_API bool isSigSoundfileBuffer(Tree s, Tree& sf, Tree& chan, Tree& part, Tree& ridx)
 {
     return isTree(s, gGlobal->SIGSOUNDFILEBUFFER, sf, chan, part, ridx);
 }
@@ -955,7 +961,20 @@ Tree listConvert(const siglist& a)
 {
     int  n = (int)a.size();
     Tree t = gGlobal->nil;
-    
+
     while (n--) t = cons(a[n], t);
     return t;
 }
+
+/*
+ * Convert a Tree in stl vector of signals
+ */
+siglist treeConvert(Tree t)
+{
+    siglist res;
+    while (!isNil(t)) {
+        res.push_back(hd(t));
+        t = tl(t);
+    }
+    return res;
+}
